using IglooCoder.Commons.Logging;
using IglooCoder.Demo.AOP.Common;
using IglooCoder.Demo.AOP.PostSharp.Infrastructure;
using IglooCoder.Demo.AOP.PostSharp.Infrastructure.PostSharpLogging;

namespace IglooCoder.Demo.AOP.PostSharp.Services
{
    public class CustomerServices : ICustomerServices
    {
        private readonly IUpdateCustomerValidator _updateCustomerValidator;
        private readonly ICustomerRepository _customerRepository;

        public CustomerServices(IUpdateCustomerValidator updateCustomerValidator, ICustomerRepository customerRepository)
        {
            _updateCustomerValidator = updateCustomerValidator;
            _customerRepository = customerRepository;
        }

        [LogThis]
        public void Update(UpdateCustomerDto updateCustomerDto)
        {
            Log.For(this)
                .Info("PostSharp.CustomerServices.Update started (old way)");

            if (!_updateCustomerValidator.For(updateCustomerDto))
            {
                Log.For(this)
                    .Info("UpdateCustomerDto failed validation (old way)");
                throw new ValidationException();
            }

            _customerRepository.SaveOrUpdate(updateCustomerDto);

            Log.For(this)
                .Info("PostSharp.CustomerServices.Update finished (old way)");
        }
    }
}